﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Excel = Microsoft.Office.Interop.Excel;
using Office = Microsoft.Office.Core;
using System.Drawing;
using Extentions;
using static PasswordCheck.PasswordChecker;

namespace PasswordCheck
{
    class AppHdlExcel
    {
        private ResultHandler resultHundler;
        private Keyword keywordHundler;
        private Excel.Application excelApp;
        private Excel.Workbook book;
        private string tagName;

        // プログレスバー更新用デリゲートとハンドラ
        private UpdateSecondaryProgressDelegate UpdateSecondaryProgressEvent;

        //*************************************************************************************************
        // コンストラクタ
        //*************************************************************************************************
        public AppHdlExcel(ResultHandler resultHdl, Keyword keywordHdl, UpdateSecondaryProgressDelegate updateSecondaryProgressHandler)
        {

            resultHundler = resultHdl;
            keywordHundler = keywordHdl;
            UpdateSecondaryProgressEvent = updateSecondaryProgressHandler;
        }

        public void Close()
        {
            try
            {
                if (excelApp != null)
                {
                    excelApp.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelApp);
                    excelApp = null;
                }
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show(e.ToString());
            }
        }

        //*************************************************************************************************
        // 呼び出し元
        //*************************************************************************************************
        public void AnalyzeFile(int i, List<string> passwords, bool searchKeyword = true)
        {
            List<PassResult> results = resultHundler.GetPassResults();

            if(excelApp == null)
            {
                excelApp = new Excel.Application();
                excelApp.Visible = false;
                excelApp.DisplayAlerts = false; //おためし
                                                //excelApp.AlertBeforeOverwriting = false;
            }

            tagName = results[i].fName; 

            //例外処理　Excelファイル以外があった場合スキップ（continue）
            switch (results[i].fType)
            {
                case FileType.Excel:
                case FileType.ExcelXML:
                    break;
                default:
                    return;
            }

            //例外処理　検索済みはスキップ（continue）
            switch (results[i].seachStatus)
            {
                case SeachResultStatus.Untouched:
                    break;
                default:
                    return;
            }

            //パスワードチェック処理及びファイルオープン
            //statusによる例外処理
            switch (results[i].passStatus)
            {
                //未定義、パスワード不明のみ検証
                case PassResultStatus.Incorrect:
                case PassResultStatus.Undefined:
                    book = PasswordCheck(results, i, passwords);
                    break;
                //特定済みはそのままオープン
                case PassResultStatus.Correct:
                case PassResultStatus.NonePassword:
                    book = OpenBook(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, results[i].pwd);
                    break;
                default:
                    return;
            }

            //ファイルが開けた時のみ検索処理
            if (book != null)
            {
                Office.MsoAutomationSecurity msoAS = excelApp.AutomationSecurity;　//マクロ実行設定退避
                excelApp.AutomationSecurity = Office.MsoAutomationSecurity.msoAutomationSecurityForceDisable; // マクロ強制無効化
                SearchKeyword(book);
                // 検索終了後ドキュメントクローズ
                book.Close(false, Type.Missing, Type.Missing);
                //book.Close(Excel.XlSaveAction.xlDoNotSaveChanges, Type.Missing, Type.Missing);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(book);
                excelApp.AutomationSecurity = msoAS; // マクロ実行設定復元

                results[i].seachStatus = SeachResultStatus.Finished; //検索済みフラグ設定
            }
        }



        //*************************************************************************************************
        // パスワード検索処理関数
        //*************************************************************************************************
        private Excel.Workbook PasswordCheck(List<PassResult> results, int Index, List<string> passwords)
        {

            Excel.Workbook tmpBook = null;
            List<string> testPasswords;


            //初回（Und）の検証はパスワードなしも検証
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {
                tmpBook = OpenBook(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, "");
                if (tmpBook != null)
                {
                    //パスワードなし
                    //result.pwd = "";
                    results[Index].passStatus = PassResultStatus.NonePassword;
                    return tmpBook;
                }
            }

            //検証用パスワードリスト（testPasswords）をセット
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {

                //初回（Undefind）保存済みパスワードリストを使用
                testPasswords = passwords;
            }
            else
            {
                //Incorectでパスワードが入力されている場合
                if (results[Index].pwd != "")
                {
                    testPasswords = new List<string>();
                    testPasswords.Add(results[Index].pwd);
                }
                else
                {//例外：パスワードの入力がなければ検証せず終了
                    return null;
                }
            }

            //検証用パスワードリストで検証
            foreach (string pwd in testPasswords)
            {
                tmpBook = OpenBook(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, pwd);
                if (tmpBook != null)
                {
                    //パスワード特定
                    results[Index].pwd = pwd;
                    results[Index].passStatus = PassResultStatus.Correct;
                    return tmpBook;
                }
            }

            //パスワード不明
            results[Index].pwd = "";
            results[Index].passStatus = PassResultStatus.Incorrect;
            return null;
        }

        //*************************************************************************************************
        // Bookファイルを開く関数
        // return:開いたDocument。パスワードが不明な場合はnull
        //*************************************************************************************************
        private Excel.Workbook OpenBook(string filePath, string password = "")
        {
            Excel.Workbook tmpBook = null;
            string testPass;
            Office.MsoAutomationSecurity msoAS = excelApp.AutomationSecurity;　//マクロ実行設定退避

            //パスワードが空の場合はテストパスワードを仮置き
            if (password != "")
            {
                testPass = password;
            }
            else
            {
                testPass = "NonPasswordTestSSMT";
            }

            //ファイルが開けるか確認
            try
            {
                excelApp.AutomationSecurity = Office.MsoAutomationSecurity.msoAutomationSecurityForceDisable; // マクロ強制無効化
                tmpBook = excelApp.Workbooks.Open(filePath, UpdateLinks: 0, ReadOnly: true, Password: testPass);

            }
            catch (System.Runtime.InteropServices.COMException e)
            {//開けない場合
                return null;
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show("Excelファイルを開く際、予期せぬエラーが発生しました。:" + e.ToString());
                return null;
            }
            finally
            {
                excelApp.AutomationSecurity = msoAS; // マクロ実行設定復元
            }
            return tmpBook;
        }

        //*************************************************************************************************
        // 検索処理関数
        //*************************************************************************************************
        private void SearchKeyword(Excel.Workbook book)
        {
            try
            {
                // ワークシートの総数を取得
                int sheetCount = book.Worksheets.Count;
                int sheetProcessed = 0;

                foreach (Excel.Worksheet sheet in book.Worksheets)
                {
                    foreach (Excel.Shape shape in sheet.Shapes)
                    {
                        this.ExtractSapeContents(shape, sheet.Name);
                    }

                    // ワークシートの処理が完了したので、処理済みのカウントをインクリメント
                    sheetProcessed++;

                    // プログレスバーの進捗を更新
                    double progress = ((double)sheetProcessed / sheetCount) * 80.0 + 20.0;
                    UpdateSecondaryProgressEvent?.Invoke((int)progress);
                }

                // 全ワークシートの処理が完了したら、プログレスバーを100%に更新
                UpdateSecondaryProgressEvent?.Invoke(100);
            }
            catch (Exception e)
            {
                // 例外処理
                System.Windows.Forms.MessageBox.Show(e.ToString());
            }
        }
        //*************************************************************************************************
        // Shape処理関数
        //*************************************************************************************************
        private void ExtractSapeContents(Excel.Shape s, String sheetName)
        {
            //s.Select(); 

            if (s.Type == Office.MsoShapeType.msoGroup)
            {// グループ化されたものなら、子要素に対し再帰呼び出し
                foreach (Excel.Shape subShape in s.GroupItems)
                {
                    ExtractSapeContents(subShape, sheetName);
                }

            }
            else if (s.Type == Office.MsoShapeType.msoCanvas)
            {// キャンバスであれば、子要素に対し再帰呼び出し
                foreach (Excel.Shape subShape in s.CanvasItems)
                {
                    ExtractSapeContents(subShape, sheetName);
                }
            }
            else
            {
                //例外：テキストフレームを持たないShape
                if (s.TextFrame == null) { return; }
                //例外：四角以外
                if (s.AutoShapeType != Office.MsoAutoShapeType.msoShapeRectangle) { return; }
                //例外：枠無し
                if (s.Line.Visible != Office.MsoTriState.msoTrue) { return; }
                //横長（アスペクト比２以上）ではない
                if (s.Width / s.Height < 2) { return; }

                try
                {
                    string text = s.TextFrame.Characters().Text;
                    //例外：テキストがNullかEmpty
                    if (String.IsNullOrEmpty(text)) { return; }

                    //標記用キーワードマッチング。なければreturn
                    string word = keywordHundler.CheckClassificationWords(text);
                    if (word == null) { return; }

                    System.Windows.Forms.Clipboard.Clear();
                    s.Copy();
                    //excelApp.Selection.Copy();

                    String fileName = resultHundler.ClipbordToImg();

                    String position = "シート名：" + sheetName;
                    bool imgEx = false; //仮置き

                    if (fileName != null)
                    {
                        imgEx = true;
                    }

                    resultHundler.AddDetectResult(fileName: tagName, keyword: word, imgExist: imgEx, imageName: fileName, positionStr: position);

                }
                catch (Exception e)
                {
                    //System.Windows.Forms.MessageBox.Show(sheetName + e.ToString());
                    return;
                }
            }
        }
    }


}