﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;


namespace PasswordCheck
{
    class AppHdlPDF
    {

        private iTextSharp.text.pdf.PdfReader pdfReader;
        private List<PassResult> results;
        private int i;

        //*************************************************************************************************
        // 呼び出し元
        //*************************************************************************************************
        public void AnalyzeFile(List<PassResult> passResults, int indexNum, List<string> passwords, bool searchKeyword = true)
        {
            results = passResults;
            i = indexNum;

            //例外処理　Excelファイル以外があった場合スキップ（continue）
            switch (results[i].fType)
            {
                case FileType.Pdf:
                    break;
                default:
                    return;
            }

            //例外処理　検索済みはスキップ（continue）
            switch (results[i].seachStatus)
            {
                case SeachResultStatus.Untouched:
                    break;
                default:
                    return;
            }

            //パスワードチェック処理及びファイルオープン
            //statusによる例外処理
            switch (results[i].passStatus)
            {
                //未定義、パスワード不明のみ検証
                case PassResultStatus.Incorrect:
                case PassResultStatus.Undefined:
                    pdfReader = PasswordCheck(results, i, passwords);
                    break;
                //特定済みはそのままオープン
                case PassResultStatus.Correct:
                case PassResultStatus.NonePassword:
                    //System.Windows.Forms.MessageBox.Show("オープン" + results[i].fName);
                    pdfReader = OpenPDF(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, results[i].pwd);
                    break;
            }

            //ファイルが開けた時のみ検索処理
            if (pdfReader != null)
            {
                SearchKeyword(pdfReader);
                // 検索終了後ドキュメントクローズ
                pdfReader.Close();
            }

        }

        //*************************************************************************************************
        // 呼び出し元
        //*************************************************************************************************
        //public void AnalyzeFiles(List<PassResult> results, List<string> passwords, bool searchKeyword = true)
        //{
        //    for (i = 0; i < results.Count(); i++)
        //    {
        //        //例外処理　Excelファイル以外があった場合スキップ（continue）
        //        switch (results[i].fType)
        //        {
        //            case FileType.Pdf:
        //                break;
        //            default:
        //                continue;
        //        }

        //        //パスワードチェック処理及びファイルオープン
        //        //statusによる例外処理
        //        switch (results[i].passStatus)
        //        {
        //            //未定義、パスワード不明のみ検証
        //            case PassResultStatus.Incorrect:
        //            case PassResultStatus.Undefined:
        //                pdfReader = PasswordCheck(results, i, passwords);
        //                break;
        //            //特定済みはそのままオープン
        //            case PassResultStatus.Correct:
        //            case PassResultStatus.NonePassword:
        //                //System.Windows.Forms.MessageBox.Show("オープン" + results[i].fName);
        //                pdfReader = OpenPDF(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, results[i].pwd);
        //                break;
        //        }

        //        //ファイルが開けた時のみ検索処理
        //        if (pdfReader != null)
        //        {
        //            SearchKeyword(pdfReader);
        //            // 検索終了後ドキュメントクローズ
        //            pdfReader.Close();
        //        }
        //    }

        //}

        //*************************************************************************************************
        // パスワード検索処理関数
        //*************************************************************************************************
        private iTextSharp.text.pdf.PdfReader PasswordCheck(List<PassResult> results, int Index, List<string> passwords)
        {
            iTextSharp.text.pdf.PdfReader tmpPDF = null;
            List<string> testPasswords;

            //初回（Und）の検証はパスワードなしも検証
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {
                tmpPDF = OpenPDF(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, "");
                if (tmpPDF != null)
                {
                    //パスワードなし
                    //result.pwd = "";
                    results[Index].passStatus = PassResultStatus.NonePassword;
                    return tmpPDF;
                }
            }

            //検証用パスワードリスト（testPasswords）をセット
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {

                //初回（Undefind）保存済みパスワードリストを使用
                testPasswords = passwords;
            }
            else
            {
                //Incorectでパスワードが入力されている場合
                if (results[Index].pwd != "")
                {
                    testPasswords = new List<string>();
                    testPasswords.Add(results[Index].pwd);
                }
                else
                {//例外：パスワードの入力がなければ検証せず終了
                    return null;
                }
            }

            //検証用パスワードリストで検証
            foreach (string pwd in testPasswords)
            {
                tmpPDF = OpenPDF(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, pwd);
                if (tmpPDF != null)
                {
                    //パスワード特定
                    results[Index].pwd = pwd;
                    results[Index].passStatus = PassResultStatus.Correct;
                    return tmpPDF;
                }
            }

            //パスワード不明
            results[Index].pwd = "";
            results[Index].passStatus = PassResultStatus.Incorrect;
            return null;
        }

        //*************************************************************************************************
        // Docファイルを開く関数
        // return:開いたPdf。パスワードが不明な場合はnull
        //*************************************************************************************************
        private iTextSharp.text.pdf.PdfReader OpenPDF(string filePath, string password = "")
        {
            try
            {
                if (password != "")
                {
                    return new iTextSharp.text.pdf.PdfReader(filePath, Encoding.UTF8.GetBytes(password));
                }
                else
                {
                    return new iTextSharp.text.pdf.PdfReader(filePath);
                }

            }
            catch (iTextSharp.text.exceptions.BadPasswordException e)
            {
                Debug.WriteLine(e.ToString());
                return null;
            }
            catch (iTextSharp.text.exceptions.InvalidPdfException e)
            {
                // PDFが新しい暗号化または署名がされている場合には非対応
                Debug.WriteLine(e.ToString());
                results[i].passStatus = PassResultStatus.Unknown;
                return null;
            }
            catch (Exception e)
            {
                Debug.WriteLine(e.ToString());
                return null;
            }
        }

        //*************************************************************************************************
        // 検索処理関数
        //*************************************************************************************************
        private void SearchKeyword(iTextSharp.text.pdf.PdfReader pdfReader)
        {

        }
    }
}