﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Office = Microsoft.Office.Core;
using System.Drawing;
using Extentions;
using static PasswordCheck.PasswordChecker;
using System.Windows.Forms;
using Microsoft.Office.Interop.PowerPoint;

namespace PasswordCheck
{
    class AppHdlPowerPoint
    {
        private ResultHandler resultHundler;
        private Keyword keywordHundler;
        private PowerPoint.Application pptApp;
        private PowerPoint.Presentations presentations;
        private PowerPoint.Presentation presentation;
        private string tagName;

        private PowerPoint.ProtectedViewWindows protectedViewWindows;
        private PowerPoint.ProtectedViewWindow protectedViewWindow;


        // プログレスバー更新用デリゲートとハンドラ
        private UpdateSecondaryProgressDelegate UpdateSecondaryProgressEvent;

        //*************************************************************************************************
        // コンストラクタ
        //*************************************************************************************************
        public AppHdlPowerPoint(ResultHandler resultHdl, Keyword keywordHdl, UpdateSecondaryProgressDelegate updateSecondaryProgressHandler)
        {
            resultHundler = resultHdl;
            keywordHundler = keywordHdl;
            UpdateSecondaryProgressEvent = updateSecondaryProgressHandler;
        }

        public void Close()
        {
            try
            {
                if (pptApp != null)
                {
                    //開いているプレゼンテーションがある場合はアプリケーションを閉じない
                    if (pptApp.Presentations.Count == 0)
                    {
                        pptApp.Quit();
                        //System.Runtime.InteropServices.Marshal.ReleaseComObject(presentations);
                        //presentations = null;
                        //System.Runtime.InteropServices.Marshal.ReleaseComObject(pptApp);
                        //pptApp = null;
                        //GC.Collect();

                    }
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(presentations);
                    presentations = null;
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(pptApp);
                    pptApp = null;

                }
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show(e.ToString());
            }
        }

        //*************************************************************************************************
        // 呼び出し元
        //*************************************************************************************************
        public void AnalyzeFile(int i, List<string> passwords, bool searchKeyword = true)
        {
            List<PassResult> results = resultHundler.GetPassResults();

            tagName = results[i].fName;

            pptApp = new PowerPoint.Application();
            protectedViewWindows = pptApp.ProtectedViewWindows;
            presentations = pptApp.Presentations;
            pptApp.DisplayAlerts = PowerPoint.PpAlertLevel.ppAlertsNone;

            //例外処理　パワポファイル以外があった場合スキップ（continue）
            switch (results[i].fType)
            {
                case FileType.PowerPoint:
                case FileType.PowerPointXML:
                    break;
                default:
                    return;
            }

            //例外処理　検索済みはスキップ（continue）
            switch (results[i].seachStatus)
            {
                case SeachResultStatus.Untouched:
                    break;
                default:
                    return;
            }

            //パスワードチェック処理及びファイルオープン
            //statusによる例外処理
            switch (results[i].passStatus)
            {
                //未定義、パスワード不明のみ検証
                case PassResultStatus.Incorrect:
                case PassResultStatus.Undefined:
                    protectedViewWindow = PasswordCheckProtectedViewWindow(results, i, passwords);
                    //presentation = PasswordCheck(results, i, passwords);
                    break;
                //特定済みはそのままオープン
                case PassResultStatus.Correct:
                case PassResultStatus.NonePassword:
                    protectedViewWindow = OpenProtectedViewWindow(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, results[i].pwd);
                    //presentation = OpenPresentation(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, results[i].pwd);
                    break;
                default:
                    return;
            }

            //ファイルが開けた時のみ検索処理
            if(protectedViewWindow != null)
            //if (presentation != null)
            {
                // プログレスバー２を２０％に更新
                UpdateSecondaryProgressEvent?.Invoke(20);

                SearchKeywordProctedViewWindow(protectedViewWindow);
                //SearchKeyword(presentation);

                // プログレスバー２を１００％に更新
                UpdateSecondaryProgressEvent?.Invoke(100);

                // 検索終了後ドキュメントクローズ
                protectedViewWindow.Close();
                System.Runtime.InteropServices.Marshal.ReleaseComObject(protectedViewWindow);
                protectedViewWindow = null;
                //presentation.Close();
                //System.Runtime.InteropServices.Marshal.ReleaseComObject(presentation);
                //presentation = null;


                results[i].seachStatus = SeachResultStatus.Finished;


            }


        }



        //*************************************************************************************************
        // パスワード検索処理関数
        //*************************************************************************************************
        private PowerPoint.Presentation PasswordCheck(List<PassResult> results, int Index, List<string> passwords)
        {
            PowerPoint.Presentation tmpPresentation = null;
            List<string> testPasswords;


            //初回（Und）の検証はパスワードなしも検証
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {
                tmpPresentation = OpenPresentation(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, "");
                if (tmpPresentation != null)
                {
                    //パスワードなし
                    //result.pwd = "";
                    results[Index].passStatus = PassResultStatus.NonePassword;
                    return tmpPresentation;
                }
            }


            //検証用パスワードリスト（testPasswords）をセット
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {

                //初回（Undefind）保存済みパスワードリストを使用
                testPasswords = passwords;
            }
            else
            {
                //Incorectでパスワードが入力されている場合
                if (results[Index].pwd != "")
                {
                    testPasswords = new List<string>();
                    testPasswords.Add(results[Index].pwd);
                }
                else
                {//例外：パスワードの入力がなければ検証せず終了
                    return null;
                }
            }

            //検証用パスワードリストで検証
            foreach (string pwd in testPasswords)
            {
                tmpPresentation = OpenPresentation(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, pwd);
                if (tmpPresentation != null)
                {
                    //パスワード特定
                    results[Index].pwd = pwd;
                    results[Index].passStatus = PassResultStatus.Correct;
                    return tmpPresentation;
                }
            }

            //パスワード不明
            results[Index].pwd = "";
            results[Index].passStatus = PassResultStatus.Incorrect;
            return null;
        }

        //*************************************************************************************************
        // パスワード検索処理関数
        //*************************************************************************************************
        private PowerPoint.ProtectedViewWindow PasswordCheckProtectedViewWindow(List<PassResult> results, int Index, List<string> passwords)
        {
            PowerPoint.ProtectedViewWindow tmpProtectedViewWindow = null;
            List<string> testPasswords;


            //初回（Und）の検証はパスワードなしも検証
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {
                tmpProtectedViewWindow = OpenProtectedViewWindow(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, "");
                if(tmpProtectedViewWindow != null)
                {
                    //パスワードなし
                    results[Index].passStatus = PassResultStatus.NonePassword;
                    return tmpProtectedViewWindow;
                }
            }


            //検証用パスワードリスト（testPasswords）をセット
            if (results[Index].passStatus == PassResultStatus.Undefined)
            {
                //初回（Undefind）保存済みパスワードリストを使用
                testPasswords = passwords;
            }
            else
            {
                //Incorectでパスワードが入力されている場合
                if (results[Index].pwd != "")
                {
                    testPasswords = new List<string>();
                    testPasswords.Add(results[Index].pwd);
                }
                else
                {//例外：パスワードの入力がなければ検証せず終了
                    return null;
                }
            }

            //検証用パスワードリストで検証
            foreach (string pwd in testPasswords)
            {
                tmpProtectedViewWindow = OpenProtectedViewWindow(ThisAddIn.WORK_FOLDER_PATH + results[Index].fName, pwd);
                if (tmpProtectedViewWindow != null)
                {
                    //パスワード特定
                    results[Index].pwd = pwd;
                    results[Index].passStatus = PassResultStatus.Correct;
                    return tmpProtectedViewWindow;
                }
            }

            //パスワード不明
            results[Index].pwd = "";
            results[Index].passStatus = PassResultStatus.Incorrect;
            return null;
        }

        //*************************************************************************************************
        // PowerPointファイルを開く関数
        // return:開いたPresentation。パスワードが不明な場合はnull
        //*************************************************************************************************
        private PowerPoint.Presentation OpenPresentation(string filePath, string password = "")
        {


            PowerPoint.Presentation tmpPresentation = null;
            string testPass;
            Office.MsoAutomationSecurity msoAS = pptApp.AutomationSecurity;　//マクロ実行設定退避

            //パスワードが空の場合はテストパスワードを仮置き
            if (password != "")
            {
                testPass = password;
            }
            else
            {
                testPass = "NonPasswordTestSSMT";
            }

            //ファイルが開けるか確認
            try
            {
                pptApp.AutomationSecurity = Office.MsoAutomationSecurity.msoAutomationSecurityForceDisable; // マクロ強制無効化
                tmpPresentation = presentations.Open(filePath + "::" + testPass + "::NonPasswordTestSSMT", ReadOnly: Office.MsoTriState.msoTrue, WithWindow: Office.MsoTriState.msoFalse);

            }
            catch(System.Runtime.InteropServices.COMException e)
            {

                //開けない場合
                if (e.ErrorCode == -2146827284)
                {
                    MessageBox.Show(e.Message); 
                }

            }
            catch (Exception e)
            {//その他のエラー
                return null;
            }
            finally
            {
                pptApp.AutomationSecurity = msoAS; // マクロ実行設定復元
            }
            return tmpPresentation;
        }

        //*************************************************************************************************
        // PowerPointファイルをProtectedViewWindowで開く関数
        // return:開いたProtectedViewWindow。パスワードが不明な場合はnull
        //*************************************************************************************************
        private PowerPoint.ProtectedViewWindow OpenProtectedViewWindow(string filePath, string password = "")
        {
            PowerPoint.ProtectedViewWindow tmpProtectedViewWindow= null;
            string testPass;
            Office.MsoAutomationSecurity msoAS = pptApp.AutomationSecurity;　//マクロ実行設定退避

            //パスワードが空の場合はテストパスワードを仮置き
            if (password != "")
            {
                testPass = password;
            }
            else
            {
                testPass = "NonPasswordTestSSMT";
            }

            //ファイルが開けるか確認
            try
            {
                pptApp.AutomationSecurity = Office.MsoAutomationSecurity.msoAutomationSecurityForceDisable; // マクロ強制無効化
                tmpProtectedViewWindow = pptApp.ProtectedViewWindows.Open(filePath, testPass, Office.MsoTriState.msoFalse);
                tmpProtectedViewWindow.WindowState = PpWindowState.ppWindowMinimized;
            }
            catch (System.Runtime.InteropServices.COMException e)
            {
                //MessageBox.Show(e.Message);
                //MessageBox.Show(e.ErrorCode.ToString());

                return null;

            }
            catch (Exception e)
            {//その他のエラー
                //MessageBox.Show(e.ToString());
                return null;
            }
            finally
            {
                pptApp.AutomationSecurity = msoAS; // マクロ実行設定復元
            }
            return tmpProtectedViewWindow;
        }

        //*************************************************************************************************
        // 検索処理関数
        //*************************************************************************************************
        private void SearchKeywordProctedViewWindow(PowerPoint.ProtectedViewWindow protectedViewWindow)
        {
            try
            {
                int slideCount = protectedViewWindow.Presentation.Slides.Count;
                for (int i = 1; i <= slideCount; i++)
                {
                    PowerPoint.Slide slide = protectedViewWindow.Presentation.Slides[i];
                    foreach (PowerPoint.Shape shape in slide.Shapes)
                    {
                        this.ExtractSapeContents(shape, i); // `i` は現在のスライド番号
                    }

                    // プログレスバーの進捗を更新
                    double progress = ((double)(i - 1) / (slideCount - 1)) * 80.0 + 20.0;
                    UpdateSecondaryProgressEvent?.Invoke((int)progress);
                }

                foreach (PowerPoint.Shape s in protectedViewWindow.Presentation.SlideMaster.Shapes)
                {
                    this.ExtractSapeContents(s, -1);
                }
            }
            catch (Exception e)
            {
                //例外（デバッグ用）
                System.Windows.Forms.MessageBox.Show(tagName + "のファイル内検索エラー\nオブジェクトの抽出に失敗しました。\n" + e.ToString());
            }
        }

        //*************************************************************************************************
        // 検索処理関数
        //*************************************************************************************************
        private void SearchKeyword(PowerPoint.Presentation presentation)
        {
            try
            {
                int slideCount = presentation.Slides.Count;
                for (int i = 1; i <= slideCount; i++)
                {
                    PowerPoint.Slide slide = presentation.Slides[i];
                    foreach (PowerPoint.Shape shape in slide.Shapes)
                    {
                        this.ExtractSapeContents(shape, i); // `i` は現在のスライド番号
                    }

                    // プログレスバーの進捗を更新
                    double progress = ((double)(i - 1) / (slideCount - 1)) * 80.0 + 20.0;
                    UpdateSecondaryProgressEvent?.Invoke((int)progress);
                }

                foreach (PowerPoint.Shape s in presentation.SlideMaster.Shapes)
                {
                    this.ExtractSapeContents(s, -1);
                }
            }
            catch (Exception e)
            {
                //例外（デバッグ用）
                System.Windows.Forms.MessageBox.Show(tagName + "のファイル内検索エラー\nオブジェクトの抽出に失敗しました。\n" + e.ToString());
            }
        }

        //*************************************************************************************************
        // Shape処理関数
        //*************************************************************************************************
        private void ExtractSapeContents(PowerPoint.Shape s, int slideNum)
        {
            //s.Select(); //pptは表示がアクティブじゃないとselectできない

            if (s.Type == Office.MsoShapeType.msoGroup)
            {// グループ化されたものなら、子要素に対し再帰呼び出し
                foreach (PowerPoint.Shape subShape in s.GroupItems)
                {
                    ExtractSapeContents(subShape, slideNum);
                }

            }
            else if (s.Type == Office.MsoShapeType.msoCanvas)
            {// キャンバスであれば、子要素に対し再帰呼び出し
                foreach (PowerPoint.Shape subShape in s.CanvasItems)
                {
                    ExtractSapeContents(subShape, slideNum);
                }
            }
            else
            {
                //例外：テキストを持たないShape
                if (s.TextFrame == null || s.TextFrame.HasText == 0) { return; }
                //例外：四角以外
                if (s.AutoShapeType != Office.MsoAutoShapeType.msoShapeRectangle) { return; }
                //例外：枠無し
                if (s.Line.Visible != Office.MsoTriState.msoTrue) { return; }
                //横長（アスペクト比２以上）ではない
                if (s.Width / s.Height < 2) {  return; }

                string text = s.TextFrame.TextRange.Text;
                //例外：テキストがNullかEmpty
                if (String.IsNullOrEmpty(text)) { return; }

                //標記用キーワードマッチング。なければreturn
                string word = keywordHundler.CheckClassificationWords(text);
                if (word == null) { return; }


                System.Windows.Forms.Clipboard.Clear();
                s.Copy();
                
                String fileName = resultHundler.ClipbordToImg();
                try
                {
                    String position;
                    if (slideNum > -1)
                    {
                        position = "スライド" + slideNum.ToString();
                    }
                    else
                    {
                        position = "スライドマスタ";
                    }
                    bool imgEx = false; //仮置き

                    if (fileName != null)
                    {
                        imgEx = true;
                    }

                    resultHundler.AddDetectResult(fileName: tagName, keyword: word, imgExist: imgEx, imageName: fileName, positionStr: position);

                }
                catch (Exception e)
                {
                    System.Windows.Forms.MessageBox.Show(e.ToString());
                }


                //Shape内テキスト判定処理

                //wordApp.Selection.Copy();
                //System.Windows.Forms.IDataObject data = System.Windows.Forms.Clipboard.GetDataObject();
                //System.IO.MemoryStream ms = (System.IO.MemoryStream)data.GetData("PNG");
                //Image.FromStream(ms).Save(ThisAddIn.WORK_FOLDER_PATH + "test.png");
            }
        }

    }
}