﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Word = Microsoft.Office.Interop.Word;
using Office = Microsoft.Office.Core;
using System.Drawing;
using Extentions;
using System.Diagnostics;
using static PasswordCheck.PasswordChecker;

namespace PasswordCheck
{
    class AppHdlWord
    {
        private ResultHandler resultHundler;
        private Keyword keywordHundler;
        private Word.Application wordApp;
        private Word.Document doc;
        private string tagName;

        // プログレスバー更新用デリゲートとハンドラ
        private UpdateSecondaryProgressDelegate UpdateSecondaryProgressEvent;

        //*************************************************************************************************
        // コンストラクタ
        //*************************************************************************************************
        public AppHdlWord(ResultHandler resultHdl, Keyword keywordHdl, UpdateSecondaryProgressDelegate updateSecondaryProgressHandler)
        {

            resultHundler = resultHdl;
            keywordHundler = keywordHdl;
            UpdateSecondaryProgressEvent = updateSecondaryProgressHandler;
        }

        //*************************************************************************************************
        // クローズ処理
        //*************************************************************************************************
        public void Close()
        {
            try
            {
                if (wordApp != null)
                {
                    wordApp.Quit(Word.WdSaveOptions.wdDoNotSaveChanges);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(wordApp);
                    wordApp = null;
                }
            }
            catch (System.Runtime.InteropServices.InvalidComObjectException ex)
            {
                // COMオブジェクトが既に解放されている場合の処理
                Debug.WriteLine("COMオブジェクトは既に解放されています: " + ex.Message);
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show(e.ToString());
            }
        }

        //*************************************************************************************************
        // 呼び出し元
        //*************************************************************************************************
        public void AnalyzeFile(int i, List<string> passwords, bool searchKeyword = true)
        {

            List<PassResult> results = resultHundler.GetPassResults();

            tagName = results[i].fName;

            wordApp = new Word.Application();
            wordApp.Visible = false;
            wordApp.DisplayAlerts = Word.WdAlertLevel.wdAlertsNone;

            //例外処理　Wordファイル以外があった場合スキップ（continue）
            switch (results[i].fType)
            {
                case FileType.Word:
                case FileType.WordXML:
                    break;
                default:
                    return;
            }

            //例外処理　検索済みはスキップ（continue）
            switch (results[i].seachStatus)
            {
                case SeachResultStatus.Untouched:
                    break;
                default:
                    return;
            }

            //パスワードチェック処理及びファイルオープン
            //statusによる例外処理
            switch (results[i].passStatus)
            {
                //未定義、パスワード不明のみ検証
                case PassResultStatus.Incorrect:
                case PassResultStatus.Undefined:
                    doc = PasswordCheck(results, i, passwords);
                    break;
                //特定済みはそのままオープン
                case PassResultStatus.Correct:
                case PassResultStatus.NonePassword:
                    //System.Windows.Forms.MessageBox.Show("オープン" + results[i].fName);
                    doc = OpenDoc(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, results[i].pwd);
                    break;
                default:
                    return;
            }

            //ファイルが開けた時のみ検索処理
            if (doc != null)
            {
                SearchKeyword(doc);
                // 検索終了後ドキュメントクローズ
                doc.Close(Word.WdSaveOptions.wdDoNotSaveChanges);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(doc);

                results[i].seachStatus = SeachResultStatus.Finished;
            }

        }

        //*************************************************************************************************
        // パスワード検索処理関数
        //*************************************************************************************************
        private Word.Document PasswordCheck(List<PassResult> results, int i, List<string> passwords)
        {

            Word.Document tmpDoc = null;
            List<string> testPasswords;


            //初回（Und）の検証はパスワードなしも検証
            if (results[i].passStatus == PassResultStatus.Undefined)
            {
                tmpDoc = OpenDoc(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, "");
                if (tmpDoc != null)
                {
                    //パスワードなし
                    //result.pwd = "";
                    results[i].passStatus = PassResultStatus.NonePassword;
                    return tmpDoc;
                }
            }

            //検証用パスワードリスト（testPasswords）をセット
            if (results[i].passStatus == PassResultStatus.Undefined) { 
                
                //初回（Undefind）保存済みパスワードリストを使用
                testPasswords = passwords;
            }
            else
            {
                //Incorectでパスワードが入力されている場合
                if (results[i].pwd != "")
                {
                    testPasswords = new List<string>();
                    testPasswords.Add(results[i].pwd);
                }
                else
                {//例外：パスワードの入力がなければ検証せず終了
                    return null;
                }
            }

            //検証用パスワードリストで検証
            foreach (string pwd in testPasswords)
            {
                tmpDoc = OpenDoc(ThisAddIn.WORK_FOLDER_PATH + results[i].fName, pwd);
                if (tmpDoc != null)
                {
                    //パスワード特定
                    results[i].pwd = pwd;
                    results[i].passStatus = PassResultStatus.Correct;
                    return tmpDoc;
                }
            }

            //パスワード不明
            results[i].pwd = "";
            results[i].passStatus = PassResultStatus.Incorrect;
            return null;
        }

        //*************************************************************************************************
        // Docファイルを開く関数
        // return:開いたDocument。パスワードが不明な場合はnull
        //*************************************************************************************************
        private Word.Document OpenDoc(string filePath, string password="")
        {

            Word.Document tmpDoc = null;
            string testPass;
            Office.MsoAutomationSecurity msoAS = wordApp.AutomationSecurity;　//マクロ実行設定退避

            //パスワードが空の場合はテストパスワードを仮置き
            if (password != "")
            {
                testPass = password;
            }
            else
            {
                testPass = "NonPasswordTestSSMT";
            }

            //System.Windows.Forms.MessageBox.Show(filePath + " " + testPass);

            //ファイルが開けるか確認
            try
            {

                wordApp.AutomationSecurity = Office.MsoAutomationSecurity.msoAutomationSecurityForceDisable; // マクロ強制無効化
                //tmpDoc = wordApp.Documents.Open(filePath, ReadOnly: false, PasswordDocument: testPass, AddToRecentFiles: false, Visible: false);
                tmpDoc = wordApp.Documents.Open(filePath, ReadOnly: true, PasswordDocument: testPass, AddToRecentFiles: false, Visible: false);
            }
            catch (System.Runtime.InteropServices.COMException e)
            {
                //パスワード誤り
                Debug.WriteLine(e.ToString());
                return null;
            }
            catch (Exception e)
            {//開けない場合
                System.Windows.Forms.MessageBox.Show("WORDを開く際、予期せぬエラーが発生しました。:" + e.ToString());
                return null;
            }
            finally
            {
                wordApp.AutomationSecurity = msoAS; // マクロ実行設定復元
            }
            return tmpDoc;
        }

        //*************************************************************************************************
        // 検索処理関数
        //*************************************************************************************************
        private void SearchKeyword(Word.Document doc)
        {
            try
            {
                int totalShapes = doc.Shapes.Count;
                int processedShapes = 0;

                //図形、テキストボックス
                foreach (Word.Shape s in doc.Shapes)
                {
                    this.ExtractSapeContents(s);
                    processedShapes++;

                    // プログレスバーの進捗を更新
                    double progress = ((double)processedShapes / totalShapes) * 100.0;
                    UpdateSecondaryProgressEvent?.Invoke((int)progress);
                }

                //表・テーブル（列１、行２までのものを検索）
                //foreach (Word.Table t in doc.Tables)
                //{
                //    t.Select();

                //    if (t.Columns.Count < 2 & t.Rows.Count < 3)
                //    {

                //        //System.IO.StreamWriter sw = new System.IO.StreamWriter(ThisAddIn.OUTLOOK_FOLDER_PATH + @"testlog.txt");
                //        //sw.Write(t.Cell(1, 1).Range.Text);
                //        //sw.Write(t.Cell(2, 1).Range.Text);
                //        //sw.Close();
                //        //t.Cell(1, 1).Range.Text.SaveAs(ThisAddIn.OUTLOOK_FOLDER_PATH + @"test.log");

                //        for (int cNum = 1; cNum <= t.Columns.Count; cNum++)
                //        {
                //            for (int rNum = 1; rNum <= t.Rows.Count; rNum++)
                //            {
                //                string text = t.Cell(rNum, cNum).Range.Text.RemoveAllSpace();
                //                text = text.Substring(0, text.Length-1);

                //                System.Windows.Forms.MessageBox.Show(text);

                //                string word = keywordHundler.CheckClassificationWords(text);

                //                System.Windows.Forms.MessageBox.Show("「" + word + "」の標記疑い");
                //                if (word != null)
                //                {
                //                    wordApp.Selection.Copy();
                //                    String fileName = resultHundler.ClipbordToImg();
                //                    // String position = GetObjectPosition(); // オブジェクトの位置を取得
                //                    String position = "ダミーページ数";
                //                    resultHundler.AddDetectResult(fileName: tagName, keyword: word, imgExist: true, imageName: fileName, positionStr: position);

                //                }

                //            }
                //        }
                //    }
                //}

            }
            catch (Exception e)
            {//例外（デバッグ用）
                System.Windows.Forms.MessageBox.Show(e.ToString());
            }

        }

        //*************************************************************************************************
        // Shape処理関数
        //*************************************************************************************************
        private void ExtractSapeContents(Word.Shape s)
        {
            s.Select(); //Selectしないとエラーが出る？

            if (s.Type == Office.MsoShapeType.msoGroup)
            {// グループ化されたものなら、子要素に対し再帰呼び出し
                foreach (Word.Shape subShape in s.GroupItems)
                {
                    ExtractSapeContents(subShape);
                }

            }
            else if (s.Type == Office.MsoShapeType.msoCanvas)
            {// キャンバスであれば、子要素に対し再帰呼び出し
                foreach (Word.Shape subShape in s.CanvasItems)
                {
                    ExtractSapeContents(subShape);
                }
            }
            else
            {
                //例外：テキストを持たないShape
                if(s.TextFrame == null || s.TextFrame.HasText == 0){ return; }
                //例外：四角以外
                if(s.AutoShapeType != Office.MsoAutoShapeType.msoShapeRectangle) { return; }
                //例外：枠無し
                if (s.Line.Visible != Office.MsoTriState.msoTrue) { return; }
                //横長（アスペクト比２以上）ではない
                if (s.Width / s.Height < 2) { return; }

                string text = s.TextFrame.TextRange.Text;
                //例外：テキストがNullかEmpty
                if (String.IsNullOrEmpty(text)) { return; }

                //標記用キーワードマッチング。なければreturn
                string word = keywordHundler.CheckClassificationWords(text);
                if (word == null) { return; }

                System.Windows.Forms.Clipboard.Clear();
                wordApp.Selection.Copy();



                String fileName = resultHundler.ClipbordToImg();
                try
                {
                    String position = "ページ数不明"; //仮置き
                    bool imgEx = false; //仮置き

                    //var page = wordApp.Selection.Information[Word.WdInformation.wdActiveEndAdjustedPageNumber];
                    var page = wordApp.Selection.Information[Word.WdInformation.wdActiveEndPageNumber];

                    if (page != null && page > -1)
                    {
                        position = page.ToString() + "ページ";
                    }

                    if (fileName != null)
                    {
                        imgEx = true;
                    }

                    resultHundler.AddDetectResult(fileName: tagName, keyword: word, imgExist: imgEx, imageName: fileName, positionStr: position);

                }
                catch (Exception e)
                {
                    System.Windows.Forms.MessageBox.Show(e.ToString());
                }


            }
        }
    }
}