﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Outlook = Microsoft.Office.Interop.Outlook;
using System.IO;
using System.IO.Compression;
using System.Drawing;
using Extentions;
using System.Diagnostics;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;
using System.Threading;
using Microsoft.Office.Interop.Outlook;

namespace PasswordCheck
{

    public class PasswordChecker
    {
        //プログレスフォーム更新用のデリゲート
        public delegate void UpdateProgressDelegate(int percent);
        public delegate void UpdateSecondaryProgressDelegate(int percent);
        public delegate void UpdateStatusDelegate(string status);
        public delegate void CloseFormDelegate();

        //プログレスフォーム更新用のイベント
        public event UpdateProgressDelegate UpdateProgressEvent;
        public event UpdateSecondaryProgressDelegate UpdateSecondaryProgressEvent;
        public event UpdateStatusDelegate UpdateStatusEvent;
        public event CloseFormDelegate CloseFormEvent;

        private List<string> passwords;

        private List<string> classificationWords;
        private List<string> secretWords;

        private ResultHandler resultHundler;
        private Keyword keywordHundler;

        private AppHdlWord wordHdl;
        private AppHdlExcel excelHdl;
        private AppHdlPowerPoint pptHdl;
        private AppHdlPDF pdfHdl;

        private Form_prog progressForm;


        //*************************************************************************************************
        // コンストラクタ
        //*************************************************************************************************
        public PasswordChecker(ResultHandler resultHdl)
        {
            resultHundler = resultHdl;
            keywordHundler = new Keyword();
            string passStr;
            try
            {
                passStr = PasswordCheck.Properties.Settings.Default.passwords;
            }
            catch
            {   //設定読み込みエラー
                passStr = "";
            }
            passwords = new List<string>(passStr.Split(' '));   //設定からパスワードリスト読み込み
            passwords.RemoveAll(s => s == "");  //空パスワード排除

            //キーワード設定
            classificationWords = new List<string>();
            classificationWords.Add("秘");
            classificationWords.Add("指定前秘");
            classificationWords.Add("特定秘密");
            classificationWords.Add("secret");

            secretWords = new List<string>();
            secretWords.Add("諸元");
        }

        //*************************************************************************************************
        // 登録パスワードの追加
        //*************************************************************************************************
        public void AddPassword(List<String> newPwd)
        {
            passwords.AddRange(newPwd);
            passwords.RemoveAll(s => s == "");  //空パスワード排除
            IEnumerable<string> setPwd = passwords.Distinct(); //重複排除
            passwords = setPwd.ToList<string>();
            Properties.Settings.Default.passwords = String.Join(" ", setPwd);   //設定へ書き込み
            Properties.Settings.Default.Save(); //設定保存
        }

        //*************************************************************************************************
        // 登録パスワード全削除
        //*************************************************************************************************
        public void ClearPassword()
        {
            passwords.Clear();
            Properties.Settings.Default.passwords = ""; //設定削除
            Properties.Settings.Default.Save(); //設定保存
        }

        //*************************************************************************************************
        // Unzip処理と検索処理のラッピング
        // results  rsultリスト
        // returns  展開した分の新規resultリスト（ResultFormで受けて追加する）
        //*************************************************************************************************
        public void StartSearch()
        {
            //プログレスフォーム生成
            progressForm = new Form_prog();
            
            //イベントハンドラ追加
            UpdateProgressEvent += progressForm.UpdateProgress;
            UpdateSecondaryProgressEvent += progressForm.UpdateSecondaryProgress;
            UpdateStatusEvent += progressForm.UpdateStatus;
            CloseFormEvent += progressForm.CloseForm;

            // 新しいスレッドをSTAモードで開始
            Thread thread = new Thread(new ThreadStart(SearchTask));
            thread.SetApartmentState(ApartmentState.STA); // STAモードを指定
            thread.Start();

            progressForm.ShowDialog();

            //// スレッドの終了を待つ
            //thread.Join();

            //イベントハンドラ削除
            UpdateProgressEvent -= progressForm.UpdateProgress;
            UpdateSecondaryProgressEvent -= progressForm.UpdateSecondaryProgress;
            UpdateStatusEvent -= progressForm.UpdateStatus;
            CloseFormEvent -= progressForm.CloseForm;

            //プログレスフォーム削除
            progressForm.Dispose();
        }

        private void SearchTask()
        {
            // プログレスバーを１０％に更新
            UpdateProgressEvent?.Invoke(10);

            // ステータステキストを更新するイベントをトリガー
            UpdateStatusEvent?.Invoke("ファイルを展開中...");

            //**************************************************
            //展開するものがなくなるまでUnzip
            //**************************************************
            while (true)
            {
                List<PassResult> addResults = Unzip();
                if (addResults.Count > 0)
                {
                    resultHundler.AddPassResults(addResults);
                }
                else
                {
                    //新規の展開なしなら離脱
                    break;
                }
            }

            //**************************************************
            //入力されたパスワード検証
            //**************************************************
            SearchFiles();

            CloseFormEvent?.Invoke();// フォームを閉じる

            Close();
        }

        //*************************************************************************************************
        // results内の未展開Zipファイル、Msgファイル、Emlファイルの展開
        // results  rsultリスト
        // returns  展開した分の新規resultリスト（ResultFormで受けて追加する）
        //*************************************************************************************************
        public List<PassResult> Unzip()
        {
            List<PassResult> results = resultHundler.GetPassResults();
            List<PassResult> tmpResult = new List<PassResult>();

            for (int i = 0; i < results.Count(); i++)
            {
                //statusによる例外処理
                switch (results[i].passStatus)
                {//未定義（初回）、パスワード保護されたZipのみ、それ以外はcontinueで次へ
                    case PassResultStatus.ProtecteZip:
                        break;
                    case PassResultStatus.Undefined:
                        break;
                    default:
                        continue;
                }

                //fileTypeによる処理
                switch (results[i].fType)
                {
                    //パスワードが設定できないファイルの種類
                    case FileType.Other:    
                        results[i].pwd = "";
                        results[i].passStatus = PassResultStatus.NonePassword;
                        continue;
                    //未知のファイルタイプ
                    case FileType.Unknown:   
                        results[i].pwd = "";
                        results[i].passStatus = PassResultStatus.Unknown;
                        continue;
                    //パスワード検証が必要な文書ファイル
                    case FileType.Excel:
                    case FileType.ExcelXML:
                    case FileType.PowerPoint:
                    case FileType.PowerPointXML:
                    case FileType.Word:
                    case FileType.WordXML:
                    case FileType.Pdf:
                        continue;
                }

                if(results[i].passStatus == PassResultStatus.ProtecteZip && results[i].pwd != "")// パスワード付きＺＩＰ再検証
                {
                    try
                    {
                        String fullZipName = ThisAddIn.WORK_FOLDER_PATH + results[i].fName;
                        String workDir = Path.GetDirectoryName(fullZipName) + "\\" + Path.GetFileNameWithoutExtension(fullZipName);
                        Directory.CreateDirectory(workDir);
                        using (Ionic.Zip.ZipFile zipFile = Ionic.Zip.ZipFile.Read(fullZipName, new Ionic.Zip.ReadOptions() { Encoding = Encoding.GetEncoding("shift_jis") }))
                        {
                            zipFile.Password = results[i].pwd;
                            zipFile.ExtractAll(workDir, Ionic.Zip.ExtractExistingFileAction.OverwriteSilently);
                        }
                        foreach (string filePath in Directory.GetFiles(workDir, "*", SearchOption.AllDirectories))
                        {
                            //Resultへの追加
                            tmpResult.Add(new PassResult(fileName: filePath.Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                                            type: resultHundler.GetFileType(filePath)));
                            //resultHundler.AddPassResult(new PassResult(fileName: filePath.Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                            //                                type: GetFileType(filePath)));
                            
                        }

                        results[i].passStatus = PassResultStatus.Unzipped;
                    }
                    catch(Ionic.Zip.BadPasswordException zipE)
                    {
                        // パスワード誤り
                        Debug.WriteLine(zipE.ToString());
                    }
                    catch (System.Exception zipE)
                    {
                        //予期せぬ例外
                        System.Windows.Forms.MessageBox.Show("パスワード付ZIP展開失敗" + "File:" + results[i].fName + "\r\n"
                                                                + "サポートしない圧縮形式かファイルが破損しています。" + "\r\n" + "\r\n"
                                                                + zipE.ToString());
                        results[i].passStatus = PassResultStatus.Unknown;
                    }

                }
                else if(results[i].fType == FileType.Zip) //初回（ResultStatus.Undfined）
                {
                    try
                    {
                        String fullZipName = ThisAddIn.WORK_FOLDER_PATH + results[i].fName;
                        String workDir = Path.GetDirectoryName(fullZipName) + "\\" + Path.GetFileNameWithoutExtension(fullZipName);
                        Directory.CreateDirectory(workDir);

                        using (Ionic.Zip.ZipFile zipFile = Ionic.Zip.ZipFile.Read(fullZipName, new Ionic.Zip.ReadOptions() { Encoding = Encoding.GetEncoding("shift_jis") }))
                        {
                            Boolean isProtected = false;
                            foreach (Ionic.Zip.ZipEntry entry in zipFile.Entries)
                            {
                                if (entry.UsesEncryption)
                                {
                                    isProtected = true;
                                    break;
                                }
                            }
                            if (isProtected)
                            {
                                //パスワードあり
                                results[i].passStatus = PassResultStatus.ProtecteZip; //パスワード有り仮置き
                                foreach (string pwd in passwords)
                                {
                                    if (pwd == "") continue; //念のため空パスワードの例外排除
                                    try
                                    {
                                        zipFile.Password = pwd;
                                        zipFile.ExtractAll(workDir, Ionic.Zip.ExtractExistingFileAction.OverwriteSilently);

                                        //エラーが出ないかったらパスワード特定、展開されたファイルを記録
                                        foreach (string filePath in Directory.GetFiles(workDir, "*", SearchOption.AllDirectories))
                                        {
                                            tmpResult.Add(new PassResult(fileName: filePath.Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                                                            type: resultHundler.GetFileType(filePath)));
                                            //resultHundler.AddPassResult(new PassResult(fileName: filePath.Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                            //                                type: GetFileType(filePath)));
                                        }
                                        results[i].pwd = pwd;
                                        results[i].passStatus = PassResultStatus.Unzipped;
                                        break;
                                    }
                                    catch (Ionic.Zip.BadPasswordException zipE)
                                    {
                                        // パスワード誤り
                                    }
                                    catch (System.Exception zipE)
                                    {
                                        //予期せぬ例外
                                        System.Windows.Forms.MessageBox.Show("パスワード付ZIP展開失敗" + "File:" + results[i].fName + "\r\n"
                                                                                + "サポートしない圧縮形式かファイルが破損しています。" + "\r\n" + "\r\n"
                                                                                + zipE.ToString());
                                        results[i].passStatus = PassResultStatus.Unknown;
                                    }
                                }
                            }
                            else
                            {
                                //パスワードなし
                                zipFile.ExtractAll(workDir, Ionic.Zip.ExtractExistingFileAction.OverwriteSilently);
                                foreach (string filePath in Directory.GetFiles(workDir, "*", SearchOption.AllDirectories))
                                {
                                    tmpResult.Add(new PassResult(fileName: filePath.Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                                                    type: resultHundler.GetFileType(filePath)));
                                    //resultHundler.AddPassResult(new PassResult(fileName: filePath.Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                    //                                type: GetFileType(filePath)));
                                }

                                results[i].passStatus = PassResultStatus.Unzipped;
                            }
                            zipFile.Password = results[i].pwd;
                        }
                    }
                    catch (System.Exception e)
                    {
                        System.Windows.Forms.MessageBox.Show("Num:" + i + "\r\n" + "File:" + results[i].fName + "\r\n" + e.ToString());
                    }
                }
                else if (results[i].fType == FileType.Eml)
                {
                    try
                    {
                        String fullMsgName = ThisAddIn.WORK_FOLDER_PATH + results[i].fName;
                        String workDir = Path.GetDirectoryName(fullMsgName) + "\\" + Path.GetFileNameWithoutExtension(fullMsgName);
                        Directory.CreateDirectory(workDir);

                        CDO.Message msg = new CDO.MessageClass();
                        ADODB.Stream stream = new ADODB.StreamClass();
                        stream.Open(Type.Missing, ADODB.ConnectModeEnum.adModeUnknown, ADODB.StreamOpenOptionsEnum.adOpenStreamUnspecified, String.Empty, String.Empty);
                        stream.LoadFromFile(fullMsgName);
                        stream.Flush();
                        msg.DataSource.OpenObject(stream, "_Stream");
                        msg.DataSource.Save();

                        foreach (CDO.IBodyPart file in msg.Attachments)
                        {
                            try
                            {
                                file.SaveToFile(workDir + "\\" + file.FileName);
                                tmpResult.Add(new PassResult(fileName: (workDir + "\\" + file.FileName).Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                                                                                     type: resultHundler.GetFileType(workDir + "\\" + file.FileName)));
                            }
                            catch (System.Exception fileSaveE)
                            {// 埋め込みオブジェクト処理の時多分エラー出る
                                tmpResult.Add(new PassResult(fileName: (workDir + "\\" + file.FileName).Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                                                                                    type: resultHundler.GetFileType(workDir + "\\" + file.FileName),
                                                                                                    rStatus: PassResultStatus.Object));
                            }
                        }

                        results[i].passStatus = PassResultStatus.Unzipped;
                    }
                    catch (System.Exception e)
                    {
                        System.Windows.Forms.MessageBox.Show("Num:" + i + "\r\n" + "File:" + results[i].fName + "\r\n" + e.ToString());
                    }
                }
                else if(results[i].fType == FileType.Msg)
                {
                    try
                    {
                        String fullMsgName = ThisAddIn.WORK_FOLDER_PATH + results[i].fName;
                        String workDir = Path.GetDirectoryName(fullMsgName) + "\\" + Path.GetFileNameWithoutExtension(fullMsgName);
                        Directory.CreateDirectory(workDir);

                        Outlook.Application oOutlookApp = new Outlook.Application();
                        Outlook.MailItem oMailItem;
                        oMailItem = oOutlookApp.CreateItemFromTemplate(fullMsgName);

                        foreach (Outlook.Attachment file in oMailItem.Attachments)
                        {
                            try
                            {
                                file.SaveAsFile(workDir + "\\" + file.FileName);
                                tmpResult.Add(new PassResult(fileName: (workDir + "\\" + file.FileName).Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                                                                                     type: resultHundler.GetFileType(workDir + "\\" + file.FileName)));
                            }
                            catch (System.Exception fileSaveE)
                            {// 埋め込みオブジェクト処理の時多分エラー出る
                                tmpResult.Add(new PassResult(fileName: (workDir + "\\" + file.FileName).Remove(0, ThisAddIn.WORK_FOLDER_PATH.Length), //作業フォルダ以下のパスのみ保存
                                                                                                    type: resultHundler.GetFileType(workDir + "\\" + file.FileName),
                                                                                                    rStatus: PassResultStatus.Object));
                            }
                        }

                        results[i].passStatus = PassResultStatus.Unzipped;
                    }
                    catch (System.Exception e)
                    {
                        System.Windows.Forms.MessageBox.Show("Num:" + i + "\r\n" + "File:" + results[i].fName + "\r\n" + e.ToString());
                    }
                }
            }
            return tmpResult;
        }

        public void SearchFiles()
        {

            List<PassResult> results = resultHundler.GetPassResults();
            for (int i = 0; i < results.Count(); i++)
            {

                // プログレスバーの進捗を更新するイベントをトリガー
                UpdateProgressEvent?.Invoke(10 + 90 * i / results.Count());

                // プログレスバー２を０％に更新
                UpdateSecondaryProgressEvent?.Invoke(0);

                // ステータステキストを更新するイベントをトリガー
                UpdateStatusEvent?.Invoke(i + "/" + results.Count() + " " + results[i].fName + "を検索中...");

                switch (results[i].fType)
                {
                    case FileType.Word:
                    case FileType.WordXML:
                        if (wordHdl == null)
                        {
                            wordHdl = new AppHdlWord(resultHundler, keywordHundler, UpdateSecondaryProgressEvent);
                        }
                        wordHdl.AnalyzeFile(i, passwords);
                        break;
                    case FileType.Excel:
                    case FileType.ExcelXML:
                        if (excelHdl == null)
                        {
                            excelHdl = new AppHdlExcel(resultHundler, keywordHundler, UpdateSecondaryProgressEvent);
                        }
                        excelHdl.AnalyzeFile(i, passwords);
                        break;
                    case FileType.PowerPoint:
                    case FileType.PowerPointXML:
                        if (pptHdl == null)
                        {
                            pptHdl = new AppHdlPowerPoint(resultHundler, keywordHundler, UpdateSecondaryProgressEvent);
                        }
                        pptHdl.AnalyzeFile(i, passwords);
                        break;
                    case FileType.Pdf:
                        if (pdfHdl == null)
                        {
                            pdfHdl = new AppHdlPDF();
                        }
                        pdfHdl.AnalyzeFile(results, i, passwords);
                        break;
                }

            }
            // プログレスバーを１００％に更新
            UpdateProgressEvent?.Invoke(100);
            // ステータステキストを更新するイベントをトリガー
            UpdateStatusEvent?.Invoke(results.Count() + "/" + results.Count() + " 検索終了");
            Thread.Sleep(1000);

        }



        public void Close()
        {
            if (wordHdl != null)
            {
                wordHdl.Close();
            }
            if (excelHdl != null)
            {
                excelHdl.Close();
            }
            if (pptHdl != null)
            {
                pptHdl.Close();
            }
            GC.Collect();
        }
    }
}
