﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing;

namespace PasswordCheck
{
    public class ResultHandler
    {
        private int passIndexNum;
        private int detectIndexNum;
        private int imageNum;
        private List<PassResult> passResults;
        private List<DetectResult> detectResults;

        //*************************************************************************************************
        // コンストラクタ
        //*************************************************************************************************
        public ResultHandler()
        {
            passIndexNum = 0;
            detectIndexNum = 0;
            imageNum = 0;
            this.passResults = new List<PassResult>();
            this.detectResults = new List<DetectResult>();
        }
        //*************************************************************************************************
        //パスワードチェック結果の新規追加（単体）
        //*************************************************************************************************
        public void AddPassResult(string fileName,
                                  int Num = -1,
                                  string password = "",
                                  PassResultStatus rStatus = PassResultStatus.Undefined,
                                  SeachResultStatus sStatus = SeachResultStatus.Untouched,
                                  FileType type = FileType.Unknown)
        {
            List<PassResult> tmpList = new List<PassResult>();
            tmpList.Add(new PassResult(fileName:fileName, Num: passIndexNum, password:password, rStatus:rStatus, sStatus:sStatus, type:GetFileType(fileName)));
            AddPassResults(tmpList);
        }

        //*************************************************************************************************
        //パスワードチェック結果の新規追加（リスト）
        //*************************************************************************************************
        public void AddPassResults(List<PassResult> addList)
        {
            foreach (PassResult addResult in addList)
            {
                addResult.indexNum = passIndexNum;
                passResults.Add(addResult);
                passIndexNum = passIndexNum + 1;
            }
        }

        //*************************************************************************************************
        //キーワード検索結果の新規追加（単体）
        //*************************************************************************************************
        public void AddDetectResult(string fileName = "",
                             string keyword = "",
                             int Num = -1,
                             bool imgExist = false,
                             string imageName = "",
                             string positionStr = "")
        {
            try
            {
                detectResults.Add(new DetectResult(fileName: fileName, keyword: keyword, Num: detectIndexNum, imgExist: imgExist, imageName: imageName, positionStr: positionStr));
            }
            catch(Exception e)
            {
                MessageBox.Show(e.ToString());
            }
            detectIndexNum = detectIndexNum + 1;
        }

        //*************************************************************************************************
        //キーワード検索結果の新規追加（単体）
        //*************************************************************************************************
        //public void AddDetectResult(DetectResult addRlt)
        //{
        //    addRlt.indexNum = detectIndexNum;
        //    detectResults.Add(addRlt);
        //    detectIndexNum = detectIndexNum + 1;
        //}

        //*************************************************************************************************
        //パスワードチェック結果をGet
        //*************************************************************************************************
        public List<PassResult> GetPassResults()
        {
            return passResults;
        }

        //*************************************************************************************************
        //パスワードチェック結果をGet
        //*************************************************************************************************
        public List<DetectResult> GetDetectResults()
        {
            return detectResults;
        }

        //*************************************************************************************************
        //フォームから入力の取得（現在、パスワードのみ）
        //*************************************************************************************************
        public void GridRowToResults(DataGridViewRow row)
        {
            for (int i = 0; i < passResults.Count; i++)
            {
                if (passResults[i].indexNum == (int)row.Cells[0].Value)
                {
                    passResults[i].pwd = (string)row.Cells[3].Value;
                }
            }
        }

        //*************************************************************************************************
        // ファイルタイプ確認（拡張子のみ）
        // fName    ファイルへのフルパス
        // returns  FileType
        //*************************************************************************************************
        public FileType GetFileType(String fName)
        {
            string extension = System.IO.Path.GetExtension(fName).ToLower();
            switch (extension)
            {
                case ".doc":
                    return FileType.Word;
                case ".docx":
                case ".docm":
                    return FileType.WordXML;
                case ".ppt":
                    return FileType.PowerPoint;
                case ".pptx":
                case ".pptm":
                    return FileType.PowerPointXML;
                case ".xls":
                    return FileType.Excel;
                case ".xlsx":
                case ".xlsm":
                case ".xlsb":
                    return FileType.ExcelXML;
                case ".pdf":
                    return FileType.Pdf;
                case ".msg":
                case ".oft":
                    return FileType.Msg;
                case ".eml":
                    return FileType.Eml;
                case ".zip":
                    return FileType.Zip;
                case ".jpg":
                case ".jpeg":
                case ".png":
                case ".bmp":
                case ".txt":
                case ".xml":
                case ".lnk":
                case ".rtf":
                case ".vbs":
                case ".ps1":
                case ".exe":
                case ".dll":
                case ".config":
                case ".manifest":
                case ".pdb":
                case ".vsto":
                    return FileType.Other;
                default:
                    return FileType.Unknown;
            }
        }

        //*************************************************************************************************
        // クリップボードの画像保存
        // returns  FilePath
        //*************************************************************************************************
        public string ClipbordToImg()
        {
            string fileName = "SaveImg" + imageNum + ".png";
            try
            {
                System.Windows.Forms.IDataObject data = System.Windows.Forms.Clipboard.GetDataObject();
                if (data == null)
                {
                    return null;
                }else if (data.GetFormats().Length == 0)
                {
                    return null;
                }

                System.Windows.Forms.Clipboard.GetImage().Save(ThisAddIn.WORK_FOLDER_PATH + fileName);

            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }
            imageNum = imageNum + 1;
            return fileName;
        }
    }
}